@@ -1,13 +1,15 @@
 #!/usr/bin/env python
 
 import argparse
+import hashlib
 import json
 import os
-import pkg_resources
 import re
 import subprocess
 import sys
 
+import pkg_resources
+
 if sys.version_info < (3, 0):
     printf("Error: This script requires Python 3.")
     sys.exit(1)
@@ -18,61 +20,17 @@ DIR = os.getcwd()
 PROJECT_NAME = os.path.basename(DIR)
 PACKAGE_JSON = "package.json"
 PACKAGE_JSON_PATH = os.path.join(DIR, PACKAGE_JSON)
+UPDATE_SCRIPT_NAME = "update.sh"
+UPDATE_SCRIPT_PATH = os.path.join(DIR, UPDATE_SCRIPT_NAME)
+BUILD_SCRIPT_NAME = "build.sh"
+BUILD_SCRIPT_PATH = os.path.join(DIR, BUILD_SCRIPT_NAME)
 
 
 def main():
     args = parse_command_line_arguments()
-
-    # Check to see if the "package.json" file exists
-    if not os.path.isfile(PACKAGE_JSON_PATH):
-        error(
-            'Failed to find the "{}" file in the current working directory.'.format(
-                PACKAGE_JSON
-            )
-        )
-
-    # Check to see if we are logged in to npm
-    completed_process = subprocess.run(
-        ["npm", "whoami"],
-        shell=True,
-        stdout=subprocess.DEVNULL,
-        stderr=subprocess.DEVNULL,
-    )
-    if completed_process.returncode != 0:
-        error(
-            'The "npm whoami" command failed, so you are probably not logged in. Try doing "npm login".'
-        )
-
-    # Update the dependencies to the latest versions
-    if not args.skip_publish:
-        printf("Updating NPM dependencies...")
-        completed_process = subprocess.run(
-            [
-                "npx",
-                "npm-check-updates",
-                "--upgrade",
-                # Don't upgrade TypeScript, as latest versions will break TSTL
-                "--reject",
-                "typescript",
-                "--packageFile",
-                PACKAGE_JSON,
-                "--loglevel",
-                "silent",
-            ],
-            shell=True,
-        )
-        if completed_process.returncode != 0:
-            error(
-                'Failed to update the "{}" dependencies to the latest versions.'.format(
-                    PACKAGE_JSON
-                )
-            )
-
-        # If we updated any dependencies, then we need to install them
-        printf("Installing NPM dependencies...")
-        completed_process = subprocess.run(["npm", "install", "--silent"], shell=True)
-        if completed_process.returncode != 0:
-            error('Failed to run "npm install".')
+    check_package_json_exists()
+    check_logged_in_to_npm()
+    update_dependencies(args)
 
     # Before we increment the version number, make sure that the program compiles
     if is_typescript_project():
@@ -90,24 +48,12 @@ def main():
         printf("Re-compiling the project...")
         compile_typescript()
 
-    git_commit_if_changes(version)
+    git_commit_if_changes(args, version)
 
-    # Publish
-    printf("Publishing to NPM...")
-    completed_process = subprocess.run(
-        [
-            "npm",
-            "publish",
-            "--access",
-            "public",
-        ],
-        shell=True,
-    )
-    if completed_process.returncode != 0:
-        error("Failed to npm publish.")
+    publish_to_npm()
 
     # Done
-    printf("Published {} version {} successfully.".format(PROJECT_NAME, version))
+    printf(f"Published {PROJECT_NAME} version {version} successfully.")
 
 
 def parse_command_line_arguments():
@@ -127,27 +73,107 @@ def parse_command_line_arguments():
         "-s",
         "--skip-increment",
         action="store_true",
-        help='do not increment the version number in the "{}" file'.format(
-            PACKAGE_JSON
-        ),
+        help=f'do not increment the version number in the "{PACKAGE_JSON}" file',
     )
 
     parser.add_argument(
         "-u",
         "--skip-update",
         action="store_true",
-        help='do not update NPM dependencies in the "{}" file'.format(PACKAGE_JSON),
+        help=f'do not update NPM dependencies in the "{PACKAGE_JSON}" file',
+    )
+
+    parser.add_argument(
+        "-c",
+        "--skip-commit",
+        action="store_true",
+        help="do not make a commit to the git repository",
     )
 
     return parser.parse_args()
 
 
+def check_package_json_exists():
+    if not os.path.isfile(PACKAGE_JSON_PATH):
+        error(
+            f'Failed to find the "{PACKAGE_JSON}" file in the current working directory.'
+        )
+
+
+def check_logged_in_to_npm():
+    completed_process = subprocess.run(
+        ["npm", "whoami"],
+        shell=True,
+        stdout=subprocess.DEVNULL,
+        stderr=subprocess.DEVNULL,
+    )
+
+    if completed_process.returncode != 0:
+        error(
+            'The "npm whoami" command failed, so you are probably not logged in. Try doing "npm login".'
+        )
+
+
+def update_dependencies(args):
+    if args.skip_update:
+        return
+
+    printf("Updating NPM dependencies...")
+
+    # Get the hash before we potentially modify the "package.json" file
+    before_hash = get_hash_of_package_json()
+
+    if os.path.isfile(UPDATE_SCRIPT_PATH):
+        completed_process = subprocess.run(
+            ["bash", UPDATE_SCRIPT_PATH],
+            shell=True,
+        )
+        if completed_process.returncode != 0:
+            error(
+                f'Failed to run the "{UPDATE_SCRIPT_NAME}" script with return code {completed_process.returncode}:\n{completed_process.stdout}'
+            )
+    else:
+        completed_process = subprocess.run(
+            [
+                "npx",
+                "npm-check-updates",
+                "--upgrade",
+                "--packageFile",
+                PACKAGE_JSON,
+                "--loglevel",
+                "silent",
+            ],
+            shell=True,
+        )
+        if completed_process.returncode != 0:
+            error(
+                f'Failed to update the "{PACKAGE_JSON}" dependencies to the latest versions.'
+            )
+
+    after_hash = get_hash_of_package_json()
+    if before_hash != after_hash:
+        # The package.json file was modified, so install the new dependencies
+        printf("Installing NPM dependencies...")
+        completed_process = subprocess.run(["npm", "install", "--silent"], shell=True)
+        if completed_process.returncode != 0:
+            error('Failed to run "npm install".')
+
+
+def get_hash_of_package_json():
+    with open(PACKAGE_JSON_PATH, "rb") as file_handle:
+        file_hash = hashlib.md5()
+        file_contents = file_handle.read()
+        file_hash.update(file_contents)
+
+        return file_hash.hexdigest()
+
+
 def get_version_from_package_json():
-    with open(PACKAGE_JSON_PATH, "r") as file_handle:
+    with open(PACKAGE_JSON_PATH, "r", encoding="utf8") as file_handle:
         package_json = json.load(file_handle)
 
     if "version" not in package_json:
-        error('Failed to find the version in the "{}" file.'.format(PACKAGE_JSON_PATH))
+        error(f'Failed to find the version in the "{PACKAGE_JSON_PATH}" file.')
 
     return package_json["version"]
 
@@ -155,7 +181,7 @@ def get_version_from_package_json():
 def increment_version(version: str):
     match = re.search(r"(.+\..+\.)(.+)", version)
     if not match:
-        error('Failed to parse the version number of "{}".'.format(version))
+        error(f"Failed to parse the version number of: {version}")
     version_prefix = match.group(1)
     patch_version = int(match.group(2))  # i.e. the third number
     incremented_patch_version = patch_version + 1
@@ -165,22 +191,26 @@ def increment_version(version: str):
 
 
 def put_version(version: str):
-    with open(PACKAGE_JSON_PATH, "r") as file_handle:
+    with open(PACKAGE_JSON_PATH, "r", encoding="utf8") as file_handle:
         package_json = json.load(file_handle)
 
     package_json["version"] = version
 
-    with open(PACKAGE_JSON_PATH, "w", newline="\n") as file_handle:
+    with open(PACKAGE_JSON_PATH, "w", encoding="utf8", newline="\n") as file_handle:
         json.dump(package_json, file_handle, indent=2, separators=(",", ": "))
         file_handle.write("\n")
 
     completed_process = subprocess.run(["npx", "sort-package-json"], shell=True)
     if completed_process.returncode != 0:
-        error('Failed to sort the "{}" file.'.format(PACKAGE_JSON))
+        error(f'Failed to sort the "{PACKAGE_JSON}" file.')
 
 
 def is_typescript_project():
-    with open(PACKAGE_JSON_PATH, "r") as file_handle:
+    with open(
+        PACKAGE_JSON_PATH,
+        "r",
+        encoding="utf8",
+    ) as file_handle:
         package_json = json.load(file_handle)
 
     return (
@@ -192,11 +222,12 @@ def is_typescript_project():
 
 
 def compile_typescript():
-    build_script_path = os.path.join(DIR, "build.sh")
-    if os.path.isfile(build_script_path):
-        completed_process = subprocess.run(["bash", build_script_path], shell=True)
+    if os.path.isfile(BUILD_SCRIPT_PATH):
+        completed_process = subprocess.run(["bash", BUILD_SCRIPT_PATH], shell=True)
         if completed_process.returncode != 0:
-            error('Failed to run the "build.sh" script.')
+            error(
+                f'Failed to run the "{BUILD_SCRIPT_NAME}" script with return code {completed_process.returncode}:\n{completed_process.stdout}'
+            )
     else:
         completed_process = subprocess.run(["rm", "-rf", "dist"], shell=True)
         if completed_process.returncode != 0:
@@ -207,7 +238,10 @@ def compile_typescript():
             error('Failed to build the project with "npx tsc".')
 
 
-def git_commit_if_changes(version):
+def git_commit_if_changes(args, version):
+    if args.skip_commit:
+        return
+
     # Check to see if this is a git repository
     completed_process = subprocess.run(
         ["git", "status"],
@@ -252,16 +286,31 @@ def git_commit_if_changes(version):
     if completed_process.returncode != 0:
         error("Failed to git push.")
 
-    printf('Pushed a commit to git for version "{}".'.format(version))
+    printf(f"Pushed a commit to git for version: {version}")
+
+
+def publish_to_npm():
+    printf("Publishing to NPM...")
+    completed_process = subprocess.run(
+        [
+            "npm",
+            "publish",
+            "--access",
+            "public",
+        ],
+        shell=True,
+    )
+    if completed_process.returncode != 0:
+        error("Failed to npm publish.")
 
 
 def error(msg):
-    printf("Error: {}".format(msg))
+    printf(f"Error: {msg}")
     sys.exit(1)
 
 
-def printf(msg):
-    print(msg, flush=True)
+def printf(*args):
+    print(*args, flush=True)
 
 
 if __name__ == "__main__":
